----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:37:48 01/11/2013 
-- Design Name: 
-- Module Name:    ipbus_dist_ram32x32 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.numeric_std.all;
use work.ipbus.all;

entity ipbus_dpram32x8b is
  port(
    clk : in std_logic;
    reset: in std_logic;
    ipbus_in: in ipb_wbus;
    ipbus_out: out ipb_rbus;

    oob_strobe: in std_logic;
    oob_addr: in std_logic_vector(7 downto 0);
    oob_write: in std_logic;
    oob_ack: out std_logic;
    oob_wdata: in std_logic_vector(31 downto 0);
    oob_rdata: in std_logic_vector(31 downto 0)
    );      
end ipbus_dpram32x8b;

architecture rtl of ipbus_dpram32x8b is
  -- Ram 32x32 is a distributed ram generated using the cor generator
  COMPONENT dpram32x8b
    PORT (
      clka : IN STD_LOGIC;
      wea : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
      addra : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      dina : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
      douta : OUT STD_LOGIC_VECTOR(31 DOWNTO 0);
      clkb : IN STD_LOGIC;
      web : IN STD_LOGIC_VECTOR(0 DOWNTO 0);
      addrb : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
      dinb : IN STD_LOGIC_VECTOR(31 DOWNTO 0);
      doutb : OUT STD_LOGIC_VECTOR(31 DOWNTO 0)
      );
  END COMPONENT;

  
  signal ack : std_logic := '0';
  signal ack2 :std_logic := '0';
  signal wea   :    STD_LOGIC_VECTOR(0 DOWNTO 0);
  signal dina  :    STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal douta :    STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal web   :    STD_LOGIC_VECTOR(0 DOWNTO 0);
  signal addrb :    STD_LOGIC_VECTOR(7 DOWNTO 0);
  signal dinb  :    STD_LOGIC_VECTOR(31 DOWNTO 0);
  signal doutb :    STD_LOGIC_VECTOR(31 DOWNTO 0); 
  --signal a   : std_logic_vector (7 downto 0);
  --signal d   : std_logic_vector (31 downto 0);
  --signal we  : std_logic;
  --signal spo :  std_logic_vector (31 downto 0);
begin

  dpram32x8b_001 : dpram32x8b
  PORT MAP (
    clka => clk,
    wea => wea,
    addra => ipbus_in.ipb_addr(7 downto 0),
    dina => dina,
    douta => douta,
    clkb => clk,
    web => web,
    addrb => addrb,
    dinb => dinb,
    doutb => doutb
  );

  web <= "0";
  dinb <= x"00000000";
  addrb <= x"00";
  ipbus_out.ipb_err <= '0';
  process (clk)
  begin
    if rising_edge(clk) then
      if ipbus_in.ipb_strobe='1' and ipbus_in.ipb_write='1' then
        dina <= ipbus_in.ipb_wdata;
        wea <= "1";
      else
        wea <= "0";
      end if;

      ipbus_out.ipb_rdata <= douta;
      ack <= ipbus_in.ipb_strobe and not ack;
      ipbus_out.ipb_ack <= ack;         -- 1 clock delay on ram readout, need
                                        -- to sync ack to that.     
    end if;
  end process;

  process (clk)
  begin
    if rising_edge(clk) then
      if oob_strobe='1' and oob_write='1' then
        dinb <= oob_wdata;
        web <= "1";
      else
        web <= "0";
      end if;

      oob__rdata <= doutb;
      ack <= oob_strobe and not ack;
      oob_ack <= ack;         -- 1 clock delay on ram readout, need
                              -- to sync ack to that.
    end if;
  end process;
  
end rtl;
