----------------------------------------------------------------------------------
-- Company: Boston Unversity EDF
-- Engineer: Christopher Woodall
-- 
-- Create Date:    16:23:43 08/10/2012 
-- Design Name: 
-- Module Name:    crc_check - Behavioral 
-- Project Name: 
-- Target Devices: XC6SLX*, but should be platform independant (not tested)
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity crc_check is
  port (
    clk      : in std_logic;
    rst_n    : in std_logic;
    enable_n : in std_logic;

    data : in std_logic_vector(7 downto 0);

    crc_o : out std_logic_vector(31 downto 0)
    );
end crc_check;  -- crc

architecture Behavioral of crc_check is
  signal crc1 : std_logic_vector (31 downto 0) := x"00000000";
  signal crc2 : std_logic_vector (31 downto 0) := x"00000000";

  signal crc : std_logic_vector(31 downto 0) := x"00000000";

  signal crc_table1 : std_logic_vector(31 downto 0);
  signal crc_table2 : std_logic_vector(31 downto 0);

  signal crc_sel1 : std_logic_vector (3 downto 0);
  signal crc_sel2 : std_logic_vector (3 downto 0);
begin

  -- Specify two table lookups into the CRC32 table.
  -- The CRC32 table used is a 16 step table, derived
  -- from the CRC32 polynomial and is standard for Ethernet.
  --
  -- Works well, no errors so far.


  crc_sel1 <= crc(3 downto 0) xor data(3 downto 0);
  crc_sel2 <= crc1(3 downto 0) xor data(7 downto 4);
  crc_lookup_tables : process (data, crc, crc1, crc2)
  begin
    case (crc_sel1) is
      when x"0"   => crc_table1 <= x"4DBDF21C";
      when x"1"   => crc_table1 <= x"500AE278";
      when x"2"   => crc_table1 <= x"76D3D2D4";
      when x"3"   => crc_table1 <= x"6B64C2B0";
      when x"4"   => crc_table1 <= x"3B61B38C";
      when x"5"   => crc_table1 <= x"26D6A3E8";
      when x"6"   => crc_table1 <= x"000F9344";
      when x"7"   => crc_table1 <= x"1DB88320";
      when x"8"   => crc_table1 <= x"A005713C";
      when x"9"   => crc_table1 <= x"BDB26158";
      when x"A"   => crc_table1 <= x"9B6B51F4";
      when x"B"   => crc_table1 <= x"86DC4190";
      when x"C"   => crc_table1 <= x"D6D930AC";
      when x"D"   => crc_table1 <= x"CB6E20C8";
      when x"E"   => crc_table1 <= x"EDB71064";
      when x"F"   => crc_table1 <= x"F0000000";
      when others => crc_table1 <= x"00000000";
    end case;

    case (crc_sel2) is
      when x"0"   => crc_table2 <= x"4DBDF21C";
      when x"1"   => crc_table2 <= x"500AE278";
      when x"2"   => crc_table2 <= x"76D3D2D4";
      when x"3"   => crc_table2 <= x"6B64C2B0";
      when x"4"   => crc_table2 <= x"3B61B38C";
      when x"5"   => crc_table2 <= x"26D6A3E8";
      when x"6"   => crc_table2 <= x"000F9344";
      when x"7"   => crc_table2 <= x"1DB88320";
      when x"8"   => crc_table2 <= x"A005713C";
      when x"9"   => crc_table2 <= x"BDB26158";
      when x"A"   => crc_table2 <= x"9B6B51F4";
      when x"B"   => crc_table2 <= x"86DC4190";
      when x"C"   => crc_table2 <= x"D6D930AC";
      when x"D"   => crc_table2 <= x"CB6E20C8";
      when x"E"   => crc_table2 <= x"EDB71064";
      when x"F"   => crc_table2 <= x"F0000000";
      when others => crc_table2 <= x"00000000";
    end case;
  end process;

  -- Calculate intermediate crc polynomials for a given data input
  crc1 <= (x"0" & crc(31 downto 4)) xor crc_table1;  --(crc(3 downto 0) xor data(3 downto 0));
  crc2 <= (x"0" & crc1(31 downto 4)) xor crc_table2;  --(crc(3 downto 0) xor data(7 downto 4));

  output_crc : process (clk)
  begin
    if rising_edge(clk) then
      -- Synchronous reset and enable
      if rst_n = '0' then
        -- Reset CRC, when we need to send off a new signal!
        crc <= x"00000000";
      elsif enable_n = '0' then
        -- output crc synchronosly
        crc <= crc2;
      end if;
    end if;
  end process;

  -- Output CRC buffer to the outside world.
  crc_o <= crc;
end Behavioral;

