----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:30:53 06/06/2012 
-- Design Name: 
-- Module Name:    led_shifter - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity led_shifter is
-- Software for driving our LED Display, which consists of a 74HC164
  port (
    -- Inputs
    leds  : in std_logic_vector(7 downto 0);
    latch : in std_logic;
    clk   : in std_logic;

    -- Outputs
    led_d   : out std_logic;
    led_clk : out std_logic  -- In future revisions there shoudl be an led_en pin hooked up to B on the shift register.

    );

end led_shifter;

architecture Behavioral of led_shifter is
  -- Latch based signals
  signal latch_buffer : std_logic_vector (2 downto 0) := "000";
  signal latch_rising : std_logic                     := '0';

  -- A Buffer for LED data once its loaded in
  signal led_buffer : std_logic_vector (7 downto 0) := x"00";

  -- Start stop, counts to keep track of our shifting.
  signal start   : std_logic             := '0';
  signal counter : unsigned (3 downto 0) := x"0";
begin
  -- Technically the 74HC164 that is being used has a minimum guranteed clock period of 5ns or 200MHz...
  -- We are running at 40.08 MHz so dividing the clock is unnecissary.
  latch_rising <= '1' when latch_buffer(1 downto 0) = "01" else '0';

  -- When we want to start shifting set clock high;
  led_clk <= counter(0) when (start = '1') else '0';

  -- The data we want to send will ways be in the MSB of 
  -- led_buffer.
  led_d <= led_buffer(0);

  -- syncronize our latch input to our clock and keep a buffer.
  -- This lets us look for a rising edge on latch, without doing
  -- anything fancy with clocks.
  sync_latch : process (clk)
  begin
    if rising_edge(clk) then
      latch_buffer <= latch_buffer(1 downto 0) & latch;
    end if;
  end process;

  shift_out : process(clk)
  begin
    -- Make the changes on the falling edge of the clock so that
    -- when the clock rises for reading the data WILL be there. This
    -- is more robust than relying on delays.
    if falling_edge(clk) then
      -- When we see a rising edge on the latch initiate our shifting
      if (latch_rising = '1') then
        counter    <= x"0";
        start      <= '1';
        -- Latch/load the data in leds into led_buffer
        led_buffer <= leds;
      elsif (start <= '1') then
        counter <= counter + 1;
        if (counter(0) = '1') then
          led_buffer <= '0' & led_buffer(7 downto 1);  -- a basic left shift by 1
        end if;

        -- Reset the system we finished our count
        if (counter = x"F") then
          start <= '0';
        end if;
      else
        start <= '0';
      end if;
    end if;
  end process;
end Behavioral;
