----------------------------------------------------------------------------------
-- Company: BU EDF
-- Engineer: Christopher Woodall
-- 
-- Create Date:    15:05:00 09/05/2012 
-- Design Name: 
-- Module Name:    L1AGenerator - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: Generates L1As with given configurations. Nothing fancy.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity L1AGenerator is
  port(
    -- Clock and Reset
    clk : in std_logic;
    rst_n : in std_logic;               -- LOW ACTIVE
    reinit_n : in std_logic;            -- LOW ACTIVE
    
    -- Default Configuration Options
    config : in std_logic_vector(1 downto 0);
    phase1_enable : in std_logic;       -- When 1 use is restricted to phase 1
                                        -- mode. Cannot act as a IPBus Slave in
                                        -- this mode.

    -- IPBus Connection
    --- ADD LATER

    -- L1A Output
    L1Aout : out std_logic
    );
end L1AGenerator;

architecture Behavioral of L1AGenerator is
  signal L1Acounter : unsigned (31 downto 0) := x"00000000";
  signal L1Acompare : unsigned (31 downto 0) := x"00000000";

  -- Different possible states of the two config pins.
  constant Phase1_Idle : std_logic_vector(1 downto 0) := "00";
  constant Phase1_1Hz : std_logic_vector(1 downto 0) := "01";
  constant Phase1_10Hz : std_logic_vector(1 downto 0) := "10";
  constant Phase1_1kHz : std_logic_vector(1 downto 0) := "11";
begin
  set_configuration: process (clk)
  begin
    if rising_edge(clk) then
      if rst_n = '0' then
        L1Acompare <= x"00000000";
      else
        -- If in phase1 mode, then we can just read off the configuration
        -- registers and set our L1Acomparison register.
        if phase1_enable = '1' then
          case config is
            when Phase1_Idle =>
              L1Acompare <= x"00000000";  
            when Phase1_1Hz=>
              -- At 40.08 MHz the number of counts required to output L1As at a
              -- frequency of 1 Hz is 40,080,000
              L1Acompare <= to_unsigned(40080000, 32);  
            when Phase1_10Hz =>
              -- At 40.08 MHz the number of counts required to output L1As at a
              -- frequency of 10 Hz is 400,800.
              L1Acompare <= to_unsigned(400800, 32);                
            when Phase1_1kHz =>
              -- At 40.08 MHz the number of counts required to output L1As at a
              -- frequency of 1 kHz is 40,080.
              L1Acompare <= to_unsigned(40080, 32);                
            when others =>
              L1Acompare <= x"00000000";                
          end case; 
        else
          -- There is currently no other case.
          -- Just output idles (0).
          L1Acompare <= x"00000000";                
        end if;
      end if;
    end if;
  end process;                         -- set_configuration

  generate_L1As: process (clk)
  begin
    if rising_edge(clk) then
      if rst_n = '0' then
        L1Acounter <= x"00000000";
        L1Aout <= '0';
      else
        -- If idle, no L1A's get sent
        if config = Phase1_Idle or phase1_enable = '0' then
          L1Aout <= '0';
        else
          -- Otherwise, keep on counting up, when you reach L1Acompare send out
          -- an L1A then reset L1Acounter.
          if L1Acounter >= L1Acompare then
            L1Acounter <= x"00000000";
            L1Aout <= '1';
          else
            L1Aout <= '0';
            L1Acounter <= L1Acounter + 1;
          end if;
        end if;
      end if;
    end if;
  end process;                         -- generate_L1As

end Behavioral;
