-------------------------------------------------------------------------------
-- Arbitrates TTS State
-------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.std_logic_misc.all;
use IEEE.Numeric_std.all;
entity tts_arbitrator is
  port (
    clk :in std_logic;
    rst_n : in std_logic;
    
    chA : in std_logic_vector(7 downto 0);
    chA_strobe: in std_logic;
    chA_err : in std_logic_vector (2 downto 0);
    
    chB :in std_logic_vector(7 downto 0);
    chB_strobe :in std_logic;
    chB_err : in std_logic_vector (2 downto 0);
    
    tts_state : out std_logic_vector (3 downto 0)
    );
end tts_arbitrator;

architecture Behavioral of tts_arbitrator is
  signal chA_state : unsigned(3 downto 0) := x"0";
  signal chB_state : unsigned(3 downto 0) := x"0";
  signal chA_err_detected : std_logic;
  signal chB_err_detected : std_logic;
begin
  -- Update chA state on strobe
  get_chA_state: process(clk)
  begin
--    if rst_n = '0' then
--      chA_state <= x"0";
    if rising_edge(clk) then
      if (chA_strobe = '1') then
        if chA(3 downto 0) = x"0" then
          if or_reduce(chA_err) = '1' then
            chA_state <= x"0";
          else
            case chA(7 downto 4) is
              when x"1"|x"2"|x"4"|x"8" =>
                chA_state <= unsigned(chA(7 downto 4));
              when others =>
                chA_state <= x"0";
            end case;     
          end if;
--          chA_state <= unsigned(chA(7 downto 4));
        end if;
      end if;
    end if;
  end process;

  -- Update chB state on strobe
  get_chB_state: process(clk)
  begin
--    if rst_n = '0' then
--      chB_state <= x"0";
    if rising_edge(clk) then  
      if (chB_strobe = '1') then
        if chB(3 downto 0) = x"0" then
          if or_reduce(chB_err) = '1' then
            chB_state <= x"0";
          else
            case chB(7 downto 4) is
              when x"1"|x"2"|x"4"|x"8" =>
                chB_state <= unsigned(chB(7 downto 4));
              when others =>
                chB_state <= x"0";
            end case;     
          end if;
        end if;
      end if;
    end if;
  end process;

  -- Update the tts_state
  update_tts_state: process (clk)
  begin
--    if rst_n = '0' then
--      tts_state <= x"0";
    if rising_edge(clk) then
      if (chA_state > chB_state) then
        tts_state <= std_logic_vector(chA_state);
      else
        tts_state <= std_logic_vector(chB_state);
      end if;
    end if;
  end process;

--  err_detected <= '0';
end Behavioral;
