----------------------------------------------------------------------------------
-- Company: Boston University EDF
-- Engineer: Christopher Woodall
-- 
-- Create Date:    12:38:52 06/05/2012 
-- Design Name: TTT - TTS/TTC Adapter Phase 2 Firmware
-- Module Name:    top_module - Behavioral
-- Project Name: TTT
-- Target Devices: XC6SLX9-TQFP144
-- Tool versions: 13.3 (lin64)
-- Description: 
--
-- Dependencies: 
--
-- Revision:
-- Revision 0.1 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use work.ipbus.all;
use work.bus_arb_decl.all;
use IEEE.STD_LOGIC_MISC.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_module is
  port (

    -- SFP Signals (TTC)
    -- SFP 0 Rx (Differential Pair)
    SFP0RxN  : in  std_logic;
    SFP0RxP  : in  std_logic;
    SFP0LOS  : in  std_logic;
    SFP0Rate : out std_logic;

    -- SFP 0 Tx (Differential Pair)
    SFP0TxN     : out std_logic;
    SFP0TxP     : out std_logic;
    SFP0TxDis   : out std_logic;
    SFP0Fault : in  std_logic;

    SFP0P   : in    std_logic;
    SFP0SDA : inout std_logic;
    SFP0SCL : out   std_logic;

    SFP1RxN  : in  std_logic;
    SFP1RxP  : in  std_logic;
    SFP1LOS  : in  std_logic;
    SFP1Rate : out std_logic;

    -- SFP 0 Tx (Differential Pair)
    SFP1TxN     : out std_logic;
    SFP1TxP     : out std_logic;
    SFP1TxDis   : out std_logic;
    SFP1Fault : in  std_logic;

    SFP1P   : in    std_logic;
    SFP1SDA : inout std_logic;
    SFP1SCL : out   std_logic;

    ---- TTS RJ-45 I/O
    ----- A IN/OUT
    TTSA_Tx : out std_logic_vector (3 downto 0);
    TTSA_Rx : in  std_logic_vector (3 downto 0);
    ----- B IN/OUT
    TTSB_Tx : out std_logic_vector (3 downto 0);
    --TTSB_Rx : in  std_logic_vector (3 downto 0);


    -- Ethernet Inteface (should change signal names)
    EthRxD      : in    std_logic_vector(3 downto 0);
    EthRxDv      : in    std_logic;
    EthRxClk     : in    std_logic;
    EthRxEr      : in    std_logic;
    EthTxD       : out   std_logic_vector(3 downto 0);
    EthTxEn      : out   std_logic;
    EthTxEr      : out   std_logic;
    EthTxClk     : in    std_logic;
    EthCrs       : in    std_logic;
    EthCol       : in    std_logic;
    EthRst_n     : out   std_logic;
    EthMdio      : inout std_logic;
    EthMdc       : out   std_logic;

    -- LED Interfaces
    led_err : out std_logic;  -- Single Error LED.
    led_clk : out std_logic;  -- Clock for 8 shift register bound LEDs
    led_d   : out std_logic;  -- Data for 8 shift register bounde LEDs

    -- Clock
    clk_in : in std_logic;                 -- Clock input from ~40MHz Oscillator
    lhc_clk : in std_logic;             -- Clock input from LHC clock input
    
    -- GPIO outputs (aka Test Outputs)
    MacSel : in std_logic_vector(3 downto 0);

--    SoftReset_N : in std_logic;

--    TriggerButton_N : in std_logic;
    -- NIM Outputs
    NimL1A : out std_logic;
    NimBC0 : out std_logic;
    NimOut0 : out std_logic;
    NimOut1 : out std_logic;

    -- NIM Inputs
    NimIn : in std_logic_vector(3 downto 0);

    -- FLASH Interfaces
    FLASH_S : out  STD_LOGIC;
    FLASH_D : out  STD_LOGIC;
    FLASH_Q : in  STD_LOGIC;
    FLASH_C : out  STD_LOGIC
    );
end top_module;

architecture Behavioral of top_module is
  signal clk : std_logic := '0';
  
  component led_shifter
    port(
      leds    : in  std_logic_vector(7 downto 0);
      latch   : in  std_logic;
      clk     : in  std_logic;
      led_d   : out std_logic;
      led_clk : out std_logic
      );
  end component;
  signal leds  : std_logic_vector (7 downto 0) := x"00";
  signal latch : std_logic                     := '0';

  signal enable_n   : std_logic := '1';
  signal rx_mac_clk : std_logic;
  signal rx_mac_clk_en : std_logic;
  
  signal rst_n : std_logic := '0';
  signal rst   : std_logic := '1';

  --testing clks
  signal rx_mac_valid     : std_logic;
  signal rx_mac_badframe  : std_logic;
  signal rx_mac_goodframe : std_logic;
  signal rx_mac_data      : std_logic_vector(7 downto 0);
  signal acked         : std_logic               := '0';
  signal tx_mac_clk_en : std_logic;
  signal tx_mac_clk    : std_logic;
  signal tx_mac_valid  : std_logic               := '0';
  signal tx_mac_ack    : std_logic;
  signal tx_mac_data   : std_logic_vector(7 downto 0);
  signal tx_packet_ctr : integer range 0 to 2000000;
  signal addr_ctr      : integer range 0 to 4096 := 0;
  signal packets_rx    : integer range 0 to 400  := 0;

  -- IPBUS
  signal locked : std_logic := '0';
  signal ipb_clk         : std_logic;


  signal ipb_master_out : ipb_wbus;
  signal ipb_master_in  : ipb_rbus;
  signal mac_addr       : std_logic_vector(47 downto 0);
  signal ip_addr        : std_logic_vector(31 downto 0);
  signal oob_moti       : trans_moti;
  signal oob_tomi       : trans_tomi;
--  signal gpio           : std_logic_vector (3 downto 0);

  -- DCM Clock Generator (80MHz clk and 4 phase 40MHz clk)
  component tts_clk_gen
    port
      (-- Clock in ports
        CLK_IN           : in     std_logic;
        -- Clock out ports
        CLK40 : out std_logic;
        --CLK40_0         : out    std_logic;
        --CLK40_90          : out    std_logic;
        --CLK40_180          : out    std_logic;
        --CLK40_270          : out    std_logic;
        CLK80          : out    std_logic;
        -- Status and control signals
        RESET             : in     std_logic;
        LOCKED            : out    std_logic
        );
  end component;

  -- TTS Control Fabric
  component tts_control
    port (
      parm    : in  std_logic_vector(255 downto 0);
      orb_clk : in  std_logic_vector(3 downto 0);
      clk80   : in  std_logic;
      tts_a   : in  std_logic_vector(3 downto 0);
      cr      : in  std_logic_vector(7 downto 0);
      ar      : in  std_logic_vector(7 downto 0);
      sr      : out std_logic_vector(7 downto 0);
      dr      : out std_logic_vector(7 downto 0);
      bx_count : in std_logic_vector(11 downto 0);
      nim_l1a : out std_logic;
      nim_ttc : out std_logic;
      ecl_bc0 : out std_logic;
      test : out std_logic_vector(1 downto 0));
  end component;

  signal parm    : std_logic_vector(255 downto 0);
  signal orb_clk : std_logic_vector(3 downto 0);

  signal clk80   : std_logic;
  signal clk320 : std_logic;
  signal clk320_n :std_logic;
  signal clk160 : std_logic;
  
  signal tts_a   : std_logic_vector(3 downto 0);
  signal cr      : std_logic_vector(7 downto 0);
  signal ar      : std_logic_vector(7 downto 0);
  signal ar_mod      : std_logic_vector(7 downto 0);
  signal NimIn0_trig : std_logic;
  signal NimIn_prev : std_logic_vector(3 downto 0);
  signal TriggerButton_trig : std_logic;
  signal sr      : std_logic_vector(7 downto 0);
  signal dr      : std_logic_vector(7 downto 0);
  signal nim_l1a : std_logic;
  signal nim_ttc : std_logic;
  signal ecl_bc0 : std_logic;
  signal clk_sys :std_logic;
  signal nim_l1a_buf : std_logic;

  signal TxBC0              : std_logic_vector(7 downto 0) := x"00";
  signal TxBC0_wr           : std_logic                    := '0';
  signal TxBC0_orbit_en     : std_logic                    := '0';
  signal TxBC0_orbit_strobe : std_logic;
  signal TxBC0_sent         : std_logic;
  signal TxBC0_orbit_ctr    : unsigned (31 downto 0)       := x"00000000";
  signal L1Acounter         : integer range 0 to 20000 := 0;

    signal TxL1A : std_logic;
  signal L1Alimit : integer range 0 to 11250;
  signal Rx0L1A    : std_logic;
  signal Rx0L1A_Clk    : std_logic;
  signal Rx1L1A    : std_logic;
  signal Rx1L1A_Clk    : std_logic;

  signal ttt_ctrl : std_logic_vector(31 downto 0) := x"00000000";
  signal ttt_bcast_cmd : std_logic_vector(31 downto 0) := x"00000000";
  signal ttt_action : std_logic_vector(31 downto 0) := x"00000000";
  signal ttt_bcast_sent : std_logic := '0';
  signal ttt_bcast_wr : std_logic := '0';
  signal ttt_bcast_ar_p : std_logic := '0';
  
  signal resetttc :std_logic;
  signal L1Ajumpers :std_logic_vector (1 downto 0) := b"10";

  signal Rx0BC0 :Std_logic_vector (7 downto 0);
  signal Rx1BC0 :Std_logic_vector (7 downto 0);
  signal Rx1BcastStrobe :Std_logic;
  signal Rx0BcastStrobe :Std_logic;  
  signal Rx0BcastErr : std_logic_vector(2 downto 0);
  signal Rx1BcastErr : std_logic_vector(2 downto 0);

  signal reset_sequence_sent : std_logic;
  
  signal ResetTTC_p : std_logic;
  signal ResetTTC_counter : integer range 0 to 10000;
  signal resetTTC_db : std_logic;
  
  signal txclk : std_logic;
  signal rxclk : std_logic;
  signal txclk_b :std_logic;
  signal rxclk_b :std_logic;
  signal rx_mac_clk_en_phase : std_logic;
  signal tx_mac_clk_en_phase : std_logic;

  signal rst_dcm_n : std_logic := '1';  
  signal l1a_test : std_logic_vector (7 downto 0);
  signal ipb_clk_n : std_logic;
  signal ipb_clk_b : std_logic;

  signal ttt_clk : std_logic;
  signal lhc_clk_b : std_logic;
  signal rdata : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_wdata            : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_rdata            : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_busy             : std_logic                     := '0';
  signal   flash_md               : std_logic                     := '0';
  signal   flash_mc               : std_logic                     := '0';
  signal   flash_mcs              : std_logic_vector(1 downto 0)  := (others => '1');

  signal Rx0BC0_strobe : std_logic;
  signal Rx1BC0_strobe : std_logic;
  signal TTSState :std_logic_vector(3 downto 0);
  signal TTSState_arb :std_logic_vector(3 downto 0);

  signal chA_bcast_single_err : unsigned(15 downto 0);
  signal chA_bcast_multi_err :unsigned(15 downto 0);
  signal chB_bcast_single_err : unsigned(15 downto 0);
  signal chB_bcast_multi_err :unsigned(15 downto 0);
  signal eth_frame_err : unsigned(31 downto 0);
  signal rst_err : std_logic := '0';
  signal ttt_action_ack : std_logic_vector(31 downto 0);
begin
  ---------------------------------------------------------------------------
  -- Setup System Clocks (approx. 320MHz, 160MHz, 80MHz and 40MHz).
  ---------------------------------------------------------------------------
  clk_dcm : entity work.clk_dcm
    port map (
      clk_in => ttt_clk,
      clkx8 => clk320,
      clkx8_n => clk320_n,
      clkx4 => clk160,
      clkx2 => clk80,
      clkx1 => clk,
--      clkx1_out => open,
      reset => '0',
      locked => locked
    );

  BUFGMUX_001: BUFGMUX
    generic map (
      CLK_SEL_TYPE => "SYNC"
      )
    port map(
      S => ttt_ctrl(2),
      I0 => clk_in,
      I1 => lhc_clk_b,
      O  =>  ttt_clk
      );
  -- Run the input lhc_clk (GCLK27) waveform through a DCM to help with jitter
  -- and allow it to be routed to BUFGMUX's which can connect to clk_in
  -- (GCLK31).
  lhc_clk_001: entity work.lhc_clk
    port map(
      CLK_IN1 => lhc_clk,
      CLK_OUT1 => lhc_clk_b,
      reset => '0',
      locked => open
    );
      
  -- Generate 4 Phase Clock (FIXME)
  orb_clk(0) <= clk;               
  --orb_clk(2) <= not clk;
  --process (clk80)
  --begin
  --  if falling_edge(clk80) then
  --    orb_clk(1) <= clk;
  --    orb_clk(3) <= not clk;
  --  end if;
  --end process;

  ---------------------------------------------------------------------------
  -- Reset Logic
  ---------------------------------------------------------------------------
  EthRst_n  <= rst_n;                    -- Sync Ethernet reset
  rst       <= not(rst_n);              -- high active reset signal

  reset_delay : process(clk)
    variable counter : unsigned (31 downto 0) := x"00000000";
  begin
    if rising_edge(clk) then
      if (ttt_ctrl(31) = '1') and (rst_n = '1') then
        counter := x"00000000";
        rst_n <= '0';
      elsif (rst_n = '0') then 
        counter := counter + 1;        
        rst_n    <= counter(10);
      end if;
    end if;
  end process;
  enable_n <= '0';


  -----------------------------------------------------------------------------
  -- NIM Settings
  -----------------------------------------------------------------------------

  nim_switch: process (ttt_ctrl(7 downto 4), Rx0L1A, nim_l1a, ecl_bc0, RX1L1A, Rx0BCastStrobe, Rx1BcastStrobe, Rx0BC0(0), Rx1BC0(0))
  begin
    case (ttt_ctrl(7 downto 4)) is
      when x"0"   => NimL1A <= nim_l1a;
                     NimBC0 <= ecl_bc0;
                     NimOut1 <= '1';
                     NimOut0 <= '1';
      when x"1"   => NimL1A <= not(Rx0L1A);
                     NimBC0 <= not(Rx0BCastStrobe and Rx0BC0(0));
                     NimOut1 <= not(Rx1L1A);
                     NimOut0 <= not(Rx1BCastStrobe and RX1BC0(0));
      when others => NimL1A  <= '1';
                     NimBC0  <= '1';                             
                     NimOut1 <= '1';
                     NimOut0 <= '1';
    end case;
  end process nim_switch;


  -----------------------------------------------------------------------------
  -- Setup Ethernet Network
  -----------------------------------------------------------------------------
  -- Set MacAddress and IPAddress
  mac_addr <= X"080030F300" & x"1" & MacSel;  -- Mac Addresses between
                                               -- 08-00-30-F3-00-10 and
                                               -- 08-00-30-F3-00-2F if all
                                               -- possible positions were
                                               -- used.

  ip_addr  <= X"c0a802" & X"2" & MacSel;  -- IP Addresses between
                                              -- 192.168.2.32 and 192.168.2.63
                                              -- Correspond directly to MAC
                                              -- Addresses
  -----------------------------------------------------------------------------
  -- Buffer & Setup IPBus Clock Using BUFIO2 to divide by 4 from the ethernet
  -- tx clk.
  -----------------------------------------------------------------------------
  BUFIO2_ipbclk : BUFIO2
    generic map (
      DIVIDE => 4, -- Set the DIVCLK divider divide-by value.
      DIVIDE_BYPASS => FALSE, -- DIVCLK output sourced from Divider (FALSE) or from I input, bypassing Divider
-- (TRUE).
      I_INVERT => FALSE,
      USE_DOUBLER => FALSE
      )
    port map (
      DIVCLK => ipb_clk_b,-- rx_mac_clk, -- 1-bit Output divided clock
      IOCLK => open, -- 1-bit Output clock
      SERDESSTROBE => open, -- 1-bit Output SERDES strobe (Clock Enable)
      I => EthRxClk -- 1-bit Clock input
      );
  BUFG_ipbclk : BUFG
    port map(
      O => ipb_clk,
      I => ipb_clk_b
      );
  ipb_clk_n <= not(ipb_clk);            -- Used for FLASH DDR

  -----------------------------------------------------------------------------
  -- GENERATE Clocks for Ethernet MAC
  --
  --    Generates tx and rx mac_clk's used later and divided by 2 from the input
  --    rx clock
  -----------------------------------------------------------------------------
  BUFIO2_rx_mac_clk : BUFIO2
    generic map (
      DIVIDE => 2, -- Set the DIVCLK divider divide-by value.
      DIVIDE_BYPASS => FALSE, -- DIVCLK output sourced from Divider (FALSE) or from I input, bypassing Divider
-- (TRUE).
      I_INVERT => FALSE,
      USE_DOUBLER => FALSE
      )
    port map (
      DIVCLK => rxclk_b,-- rx_mac_clk, -- 1-bit Output divided clock
      IOCLK => open, -- 1-bit Output clock
      SERDESSTROBE => open, -- 1-bit Output SERDES strobe (Clock Enable)
      I => EthRxClk -- 1-bit Clock input
      );

  BUFG_rx_mac_clk : BUFG
    port map(
      O => rx_mac_clk,
      I => rxclk_b
      );

  BUFIO2_tx_mac_clk : BUFIO2
    generic map (
      DIVIDE => 2, -- Set the DIVCLK divider divide-by value.
      DIVIDE_BYPASS => FALSE, -- DIVCLK output sourced from Divider (FALSE) or from I input, bypassing Divider
-- (TRUE).
      I_INVERT => FALSE,
      USE_DOUBLER => FALSE
      )
    port map (
      DIVCLK => txclk_b,--tx_mac_clk, -- 1-bit Output divided clock
      IOCLK => open, -- 1-bit Output clock
      SERDESSTROBE => open, -- 1-bit Output SERDES strobe (Clock Enable)
      I => EthTxClk -- 1-bit Clock input
      );

    BUFG_tx_mac_clk : BUFG
    port map(
      O => tx_mac_clk,
      I => txclk_b
      );

  -----------------------------------------------------------------------------
  -- Generate Rx/Tx Clock Enables for Ethernet
  --
  --    Generates Rx/Tx clock enables for the ethernet MAC, these are used in
  --    lieu of a divided by 2 clock in places as to avoid cluttering the clock
  --    network with unneeded clocks and skew. They are also phase corrected
  -----------------------------------------------------------------------------
  generateClkEnablesRx: process (EthRxClk)
    variable p : std_logic;
  begin
    if rising_edge(EthRxClk) then
      if rx_mac_clk_en_phase = '0' and p = '0' then
        p := '1';
      else
        rx_mac_clk_en <= not rx_mac_clk_en;
        p := '0';
      end if;
    end if;
  end process;

  fixClkEnablesRxPhase: process (rx_mac_clk)
  begin
    if rising_edge(rx_mac_clk) then
      if rx_mac_clk_en = '0' then
        rx_mac_clk_en_phase <= '0';       
      else
        rx_mac_clk_en_phase <= '1';
      end if;
    end if;
  end process;
  

  generateClkEnablesTx: process (EthTxClk)
    variable p : std_logic;
  begin
    if rising_edge(EthTxClk) then
     if tx_mac_clk_en_phase = '0' and p = '0' then
       p := '1';
     else
        tx_mac_clk_en <= not tx_mac_clk_en;
       p := '0';
     end if;
    end if;
  end process;
  
  fixClkEnablesTxPhase: process(tx_mac_clk)
  begin
    if rising_edge(tx_mac_clk) then
      if tx_mac_clk_en = '0' then
        tx_mac_clk_en_phase <= '0';       
      else
        tx_mac_clk_en_phase <= '1';
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- Ethernet MAC
  -----------------------------------------------------------------------------
  OEthMac001 : entity work.ethernet_mac
    port map (
    rxd              => EthRxD,
    rxdv             => EthRxDv,
    rxclk            => EthRxClk,
--    rxclk => rxclk,
    
    rxer             => EthRxEr,
    txd              => EthTxD,
    txen             => EthTxEn,
    txer             => EthTxEr,
    txclk            => EthTxClk,
    crs              => EthCrs,
    col              => EthCol,
    mdio             => EthMdio,
    mdc              => EthMdc,

    rx_mac_clk       => rx_mac_clk,
    rx_mac_clk_en    => rx_mac_clk_en,
    rx_mac_valid     => rx_mac_valid,
    rx_mac_data      => rx_mac_data,
    rx_mac_goodframe => rx_mac_goodframe,
    rx_mac_badframe  => rx_mac_badframe,

    tx_mac_clk       => tx_mac_clk,
    tx_mac_clk_en    => tx_mac_clk_en,
    tx_mac_valid     => tx_mac_valid,
    tx_mac_ack       => tx_mac_ack,
    tx_mac_data      => tx_mac_data,
    rst_n            => rst_n,
    enable_n         => enable_n
    );

  -----------------------------------------------------------------------------
  -- ipbus control logic
  --
  --    Note: IPBus reset signals are high active, not low active...
  -----------------------------------------------------------------------------
  
  ipbus : entity work.ipbus_ctrl
    generic map(NOOB => 1)
    port map(
      ipb_clk         => ipb_clk,
      rst_ipb         => rst,
      rst_macclk      => rst,
      mac_txclk       => tx_mac_clk,
      mac_rxclk       => rx_mac_clk,
      mac_rxd         => rx_mac_data,
      mac_rxdvld      => rx_mac_valid,
      mac_rxgoodframe => rx_mac_goodframe,
      mac_rxbadframe  => rx_mac_badframe,
      mac_txd         => tx_mac_data,
      mac_txdvld      => tx_mac_valid,
      mac_txack       => tx_mac_ack,
      ipb_out         => ipb_master_out,
      ipb_in          => ipb_master_in,
      mac_addr        => mac_addr,
      ip_addr         => ip_addr,
      oob_moti_bus(0) => oob_moti,
      oob_tomi_bus(0) => oob_tomi
      );

  -----------------------------------------------------------------------------
  --ipbus slaves live in the entity below, and can expose top-level ports
  --The ipbus fabric is instantiated within.
  -----------------------------------------------------------------------------
  slaves : entity work.slaves port map(
    ipb_clk  => ipb_clk,
    rst      => rst,
    ipb_in   => ipb_master_out,
    ipb_out  => ipb_master_in,
    -- Top level ports from here
    parm => parm,
    cr => cr,
    ar => ar,
    dr => dr,
    sr => sr,
    l1a_test => l1a_test,
    oob_moti => oob_moti,
    oob_tomi => oob_tomi,

    flash_busy  => flash_busy,
    flash_wdata => flash_wdata,
    flash_rdata => flash_rdata,

    ttt_ctrl => ttt_ctrl,
    ttt_action => ttt_action,
--    ttt_action_ack => "00" & rst_err & x"0000000" & "0",
    ttt_action_ack => ttt_action_ack,
    ttt_bcast_cmd => ttt_bcast_cmd,

    chA_bcast_err_counter => std_logic_vector(chA_bcast_multi_err) & std_logic_vector(chA_bcast_single_err),
    chB_bcast_err_counter => std_logic_vector(chB_bcast_multi_err) & std_logic_vector(chB_bcast_single_err),

    eth_frame_err => std_logic_vector(eth_frame_err),
    MacSel => MacSel
    );
  
  led_err <= locked; --or and_reduce(cr & ar & parm & '0');
  ttt_action_ack(29) <= rst_err;
  -----------------------------------------------------------------------------
  -- Error Counter Modules
  -----------------------------------------------------------------------------
  trig_rst_err : process (ipb_clk)
  begin
    if rising_edge(ipb_clk) then
      if (ttt_action(29) = '1') then
        rst_err <= '1';
      else
        rst_err <= '0';
      end if;
    end if;
  end process;

  chAfiber_count_errors: process (clk, rst_n, rst_err )
  begin
    if rst_n = '0' or rst_err ='1' then
      chA_bcast_single_err <= x"0000";
      chA_bcast_multi_err <= x"0000";
    elsif rising_edge(clk) then
      if ((Rx0BcastErr(1) or Rx0BcastErr(2)) = '1') then
        chA_bcast_multi_err <= chA_bcast_multi_err + 1;
      elsif (Rx0BcastErr(0) = '1') then
        chA_bcast_single_err <= chA_bcast_single_err + 1;
      end if;
    end if;
  end process;
  
  chBfiber_count_errors: process (clk, rst_n, rst_err)
  begin
    if rst_n = '0' or rst_err = '1'then
      chB_bcast_single_err <= x"0000";
      chB_bcast_multi_err <= x"0000";
    elsif rising_edge(clk) then
      if ((Rx1BcastErr(1) or Rx1BcastErr(2)) = '1') then
        chB_bcast_multi_err <= chB_bcast_multi_err + 1;          
      elsif (Rx1BcastErr(0) = '1') then
        chB_bcast_single_err <= chB_bcast_single_err + 1;

      end if;      
    end if;
  end process;  

  eth_count_errors: process (rx_mac_clk, rst_n, rst_err)
  begin
    if rst_n = '0' or rst_err = '1' then
      eth_frame_err <= x"00000000";
    elsif rising_edge(rx_mac_clk) then
      if (rx_mac_badframe = '1') then
        eth_frame_err <= eth_frame_err + 1;
      end if;
    end if;
  end process;
  -----------------------------------------------------------------------------
  -- LED Display  
  -----------------------------------------------------------------------------
  LEDShift : led_shifter
    port map (
      leds    => leds,
      latch   => latch,
      clk     => clk,
      led_d   => led_d,
      led_clk => led_clk);

  led_display : process (clk)
    variable counter         : unsigned (31 downto 0) := x"00000000";
    variable pattern_counter : integer range 0 to 7   := 0;
  begin
    if rising_edge(clk) then
--      if (rst_n = '1') then
        counter := counter + 1;
        latch   <= counter(10);
--        leds <= Rx0BC0(7 downto 4) & MacSel;
        leds <= TTSState & MacSel;
--      end if;
    end if;
  end process;
  
  -----------------------------------------------------------------------------
  -- TTS Trigger and BC0 generator.
  -----------------------------------------------------------------------------
  -- Weird ordering for TTS Subsection written by Eric Hazen:
  --    OVF = 0001 => 0100 
  --    SYN = 0010 => 1000
  --    BSY = 0100 => 0001
  --    RDY = 1000 => 0010
  process (clk)
  begin
    if rising_edge(clk) then
      if (Rx0BC0(3 downto 0) = x"0") then
        TTSA_Tx <= Rx0BC0(7 downto 4);
      end if;

      if (Rx1BC0(3 downto 0) = x"0") then
        TTSB_Tx <= Rx1BC0(7 downto 4);    
      end if;
    end if;
  end process;

  TriggerButton_trig <= '0';

  tts_control_1: tts_control
    port map (
      parm    => parm,
      orb_clk => orb_clk,
      clk80   => clk80, 
      tts_a   => TTSState,
      cr      => cr,
      ar      => ar,
      sr      => sr,
      dr      => dr,
      nim_l1a => nim_l1a_buf,
      nim_ttc => nim_ttc,
      bx_count => std_logic_vector(TxBC0_orbit_ctr(11 downto 0)),
      ecl_bc0 => ecl_bc0,
      test    => open);

  nim_l1a <= nim_l1a_buf and NimIn0_trig;
  -----------------------------------------------------------------------------
  -- Nim Input Handlers
  -----------------------------------------------------------------------------
  NimIn0_handler: process (clk)
  begin
    if rising_edge(clk) then
      NimIn_prev(0) <= NimIn(0);
      if (NimIn(0) = '0') and (NimIn_prev(0) = '1') then
        NimIn0_trig <= '0';
      else
        NimIn0_trig <= '1';
      end if;
    end if;
  end process;

  -----------------------------------------------------------------------------
  -- TTS State Arbitration
  -----------------------------------------------------------------------------

  arbitrator_1: entity work.tts_arbitrator
    port map (
      clk => clk,
      rst_n => rst_n,
      
      chA_err => Rx0BcastErr,
      chA => RX0BC0(7 downto 0),
      chA_strobe => Rx0BCastStrobe,
 
      chB_err => Rx1BcastErr,           -- Channel B Broadcast Command Errors
      chB => RX1BC0(7 downto 0),        -- Channel B Broadcast Command
      chB_strobe => RX1BCastStrobe,     -- Channel B Broadcast Command Strobe

      tts_state => TTSState_arb             -- Arbitrated TTS State
      );

--  TTSState <= TTSState_arb when ttt_ctrl(8) = '0' else TTSA_Rx;
  process (clk)
  begin
    if rising_edge(clk) then
      if ttt_ctrl(8) = '1' then
        TTSState <= TTSA_Rx;
      else
        TTSState <= TTSState_arb;
      end if;
    end if;
  end process;
  -----------------------------------------------------------------------------
  -- Fiber Optic Interface (SFP)
  -----------------------------------------------------------------------------
  SFP1SDA <= '0';
  SFP1SCL <= '0';
  SFP1TxDis <= '0';
  SFP1Rate <= '0';

  -- Instantiate Module and signals
  -- Also can drive multiple DCMs from clk_in if I instantatiate my own IBUFG
  TTCInterface_001: entity work.TTCInterface
    port map (
      clk80 => clk80,
      clk320_n => clk320_n,
      clk320 => clk320,
      clk160 => clk160,
      clk40 => clk,
      rst_n              => rst_n,
      enable_n          => '0',
      TxL1A              => TxL1A,
      TxBC0              => TxBC0,
      TxBC0_wr           => TxBC0_wr,
      TxBC0_sent         => TxBC0_sent,
      TxBC0_orbit_strobe => TxBC0_orbit_strobe,
      TxBC0_orbit_en     => TxBC0_orbit_en,
      TxBC0_orbit_ctr    => TxBC0_orbit_ctr,

      Rx0L1A              => Rx0L1A,
      Rx0L1A_clk          => Rx0L1A_clk,
      Rx1L1A              => Rx1L1A,
      Rx1L1A_clk          => Rx1L1A_clk,
      Rx0BC0              => Rx0BC0,
      RX0BroadcastStrobe  => Rx0BcastStrobe,
      Rx0BC0Err           => Rx0BcastErr,
      RX1BroadcastStrobe  => Rx1BcastStrobe,
      Rx1BC0              => Rx1BC0,
      Rx1BC0Err           => Rx1BcastErr,

      SFP0RxN            => SFP0RxN,
      SFP0RxP            => SFP0RxP,
      SFP1RxN            => SFP1RxN,
      SFP1RxP            => SFP1RxP,


      SFP_LOS            => SFP0LOS,
      SFP_Rate           => SFP0Rate,
      SFP0TxN            => SFP0TxN,
      SFP0TxP            => SFP0TxP,
      SFP1TxN            => SFP1TxN,
      SFP1TxP            => SFP1TxP,

      --SFP0TxDis          => SFP0TxDis,
      --SFP0Fault          => SFP0Fault,
      --SFP0P              => SFP0P,
      --SFP0SCL            => SFP0SCL,
      --SFP0SDA            => SFP0SDA
      --SFP0LOS            => SFP0LOS,
      --SFP0Rate           => SFP0Rate,

      --SFP1TxDis          => SFP1TxDis,
      --SFP1Fault          => SFP1Fault,
      --SFP1P              => SFP1P,
      --SFP1SCL            => SFP1SCL,
      --SFP1SDA            => SFP1SDA
      --SFP1LOS            => SFP1LOS,
      --SFP1Rate           => SFP1Rate,

      SFP_TxDis          => SFP0TxDis,
      SFP_Fault          => SFP0Fault,
      SFP_P              => SFP0P,
      SFP_SCL            => SFP0SCL,
      SFP_SDA            => SFP0SDA
    );

  -----------------------------------------------------------------------------
  -- Send Broadcast Commands
  -----------------------------------------------------------------------------
  gen_bcast_wr: process(clk)
  begin
    if rising_edge(clk) then
      if TxBC0_sent = '1' then
        ttt_bcast_sent <= '1';
        TxBC0_wr <= '0';
      elsif (ttt_action(1) = '1' and ttt_bcast_sent = '0') then
        TxBC0 <= ttt_bcast_cmd(7 downto 0);
        TxBC0_wr <= '1';
        ttt_bcast_sent <= '0';
      elsif ttt_action(1) = '0' then
        ttt_bcast_sent <= '0';
      end if;
    end if;
  end process;
  
  TxBC0_orbit_en <= not(ttt_ctrl(3));
  resetTTC <= '0';

  TxL1A <= not(nim_l1a);

  sync_sent: entity work.synch port map (
    clk => ipb_clk,
    rst => rst_n,
    s => TxBC0_sent,
    y => ttt_action_ack(1)
    );
  ---------------------------------------------------------------------------
  -- Implementing SPI Flash for Programming (R/W)
  ---------------------------------------------------------------------------
  process (ipb_clk)
  begin
    if (rising_edge(ipb_clk)) then
      FLASH_S <= not flash_MCS(1) and not flash_MCS(0);
      FLASH_D <= flash_MD;
    end if;
  end process;
  
  i_flash : entity work.flash port map(
    clk         => ipb_clk,
    we          => ipb_master_out.ipb_write,
    strobe      => ipb_master_out.ipb_strobe,
    w_data      => ipb_master_out.ipb_wdata,
    addr        => ipb_master_out.ipb_addr,
    flash_wdata => flash_wdata,
    flash_rdata => flash_rdata,
    flash_busy  => flash_busy,
    MQ          => FLASH_Q,
    MD          => flash_MD,
    MCS         => flash_MCS
    );

  -- Wu's Flash Module
  i_FLASH_C_ddr : ODDR2
    port map (
      Q  => FLASH_C,                    -- 1-bit output data
      C0 => ipb_clk,                    -- 1-bit clock input
      C1 => ipb_clk_n,                  -- 1-bit clock input
      CE => '1',                        -- 1-bit clock enable input
      D0 => '0',           -- 1-bit data input (associated with C0)
      D1 => flash_MCS(1),  -- 1-bit data input (associated with C1)
      R  => '0',                        -- 1-bit reset input
      S  => '0'                         -- 1-bit set input
      );

end Behavioral;

